<?php



/**
 * Written by David Smith 2006-2008
 * nopeer@gmail.com
 **/




define( 'INVALID_PARAM', 100 );
define( 'FAILED_CALL', 101 );
define( 'QUERY_FAILED', 102 );
define( 'NO_ROWS', 103 );
define( 'OUT_OF_RANGE', 104 );
define( 'DISCONNECTED', 105 );




/**
 * @class MysqlDBException
 * @brief Simply an exception
 **/

class MysqlDBException extends Exception
{
  function __construct( $description, $code )
  {
    parent::__construct( $description, $code );
  }
};


/**
 * @class MysqlDB
 *
 * This class encapsulates a mysql database connection.
 **/

class MysqlDB
{
  var $link;
  var $last_result;
  var $last_query;
  var $debug;
  var $last_dataset;
  var $last_dataset_index;



  //! Constructor. Initialize defaults.

  function __construct()
  {
    $this->link = null;
    $this->last_query = null;
    $this->last_result = null;
    $this->last_dataset = Array();
    $this->debug = 0;
  }




  //! Destructor. close the connection if it exists

  function __destruct()
  {
    if( $this->link != null )
      mysql_close( $this->link );
  }


  /**
   * @brief                 Connects to the database using the values given
   * @param target_host     The server with the mysql server
   * @param target_user     The user to login to the database as
   * @param target_password The password for the database user
   * @param target_db       The database we want to open on the server
   * @throws e              Throw an exception if we failed to connect to the server or couldn't
   *                        select the database we wanted
   **/

  function connect( $target_host, $target_user, $target_password, $target_db )
  {
    if( ( $this->link = mysql_connect( $target_host, $target_user, $target_password ) ) === false )
        throw new MysqlDBException( 'Failed to connect to host', FAILED_CALL );

    if( mysql_select_db( $target_db, $this->link ) === false )
        throw new MysqlDBException( 'Failed to select database ' . $target_db, FAILED_CALL );
  }



  /**
   *    @brief     Makes sure we are connected to the remote host
   *    @throws e  An exception if we are not connected to a remote host.
   **/

  function EnsureConnected()
  {
    if( !$this->link )
        throw new MysqlDBException( 'Not connected to database', DISCONNECTED );
  }



  /**
    *    @brief       Performs a query using the sql given
    *    @param sql   The query in SQL
    *    @return      Nothing
    *    @throws e    QUERY_FAILED, FAILED_CALL
    *
    *    Perform the sql query given, don't throw an exception if there are no rows returned,
    *    simply set last_dataset as an empty array. Throw an exception for any other error.
    **/

  function DoQuery( $sql )
  {
    if( $this->debug )
      echo 'DoQuery( ' . $sql . ') <br>';

    $this->EnsureConnected();

    $this->last_query = $sql;
    if( ( $this->last_result = mysql_query( $sql ) ) === false )
      throw new MysqlDBException( 'Query Failed: ' . $sql . ' : ' . mysql_error(), QUERY_FAILED );

    if( $this->last_result !== true )
    {
      $new_dataset = Array();

      $num_records = $this->FetchNumRows();

      for( $ind = 0; $ind < $num_records; $ind ++ )
      {
        if( ( $new_row = mysql_fetch_array( $this->last_result, MYSQL_ASSOC ) ) === false )
          throw( new MysqlDBException( 'Failed to fetch row ' . $this->last_query, 5 ) );

        $new_dataset[] = $new_row;
      }
      $this->last_dataset = $new_dataset;
      $this->last_dataset_index = 0;
    }
  }






  /**
   * @brief             Checks that the correct number of rows have been returned
   * @param comparison  The comparison to perform {equals, lessthan or greaterthan}
   * @param num         The value to compare against
   * @throws e          Throw an exception if the condition is not met.
   **/

  function EnsureNumRows( $comparison, $num )
  {
    $this->EnsureConnected();

    $num_rows = mysql_num_rows( $this->last_result );

    if( $comparison != 'equals' && $comparison != 'lessthan' && $comparison != 'greaterthan' )
      throw new MysqlDBException( 'Unknown comparison ' . $comparison, INVALID_PARAM );

    if( $comparison == 'equals' && $num != $num_rows )
      throw new MysqlDBException( 'Wrong number of rows returned' . $this->last_query, OUT_OF_RANGE );
    else if( $comparison == 'lessthan' && $num_rows >= $num )
      throw new MysqlDBException( 'Wrong number of rows returned' . $this->last_query, OUT_OF_RANGE );
    else if( $comparison == 'greaterthan' && $num_rows <= $num )
      throw new MysqlDBException( 'Wrong number of rows returned' . $this->last_query, OUT_OF_RANGE );
  }



  /**
   *    @brief    Fetch the number or rows that were returned by the last result.
   *    @returns  The number of rows in the last result
   **/

  function FetchNumRows()
  {
    $this->EnsureConnected();

    $num_rows = mysql_num_rows( $this->last_result );

    return $num_rows;
  }



  /**
   *    @brief    Fetch the next row from the result of the last query
   *    @returns  An associative array containing the record fetched
   *    @throws e An exception if we have gone beyond the number of rows returned
   **/

  function FetchRowAsArray()
  {
    if( $this->last_dataset_index >= $this->FetchNumRows() )
      throw new MysqlDBException( 'fetching row out of range', OUT_OF_RANGE );

    return $this->last_dataset[ $this->last_dataset_index ++ ];
  }



  /**
   *    @brief     - Gets the id of the tuple inserted during the last insert query.
   *    RECEIVES - Nothing.
   *    @returns  - The last insert id.
   *    @throws   - An excpetion for any sql errors encountered.
   **/

  function GetInsertID()
  {
    $this->EnsureConnected();

    if( ( $id = mysql_insert_id( $this->link ) ) === false )
      throw new MysqlDBException( 'Unable to retreive insert id', FAILED_CALL );

    return $id;
  }



  /**
    *   @brief     - Retreives the tuples from the last result and returns them in an array
    *   @returns  - An array of tuples containing the last result
    *   @throws   - Any database exception.
    **/

  function GetLastResult()
  {
    return $this->last_dataset;
  }





  /**
   * @brief             Selects all the rows from the table given that have fields matching the pairs in the array
   * @param table_name  The name of the table to select from
   * @param field_specs An associative array of fields to form the WHERE clause from
   * @returns           An array of tuples from the table
   * @throws e          Any database exception
   * @example e         $row = SelectByFields( 'users', Array( 'name' => 'nopeer' ) );
   **/

  function SelectByFields( $table_name, $field_specs )
  {
    $select_sql = ' SELECT * FROM ' . $table_name . ' WHERE ';

    $first = true;
    foreach( $field_specs as $next_field => $next_value )
    {
      if( !$first )
        $select_sql .= ' AND ';
      $select_sql .= $next_field . ' = ';
      if( !is_numeric( $next_value ) )
        $select_sql .= '\'';

      $select_sql .= $next_value;

      if( !is_numeric( $next_value ) )
        $select_sql .= '\'';

      $first = false;
    }

    if( $this->debug )
      echo $select_sql . ' <br> ';

    $this->DoQuery( $select_sql );

    return $this->GetLastResult();
  }



  /**
   *   @brief   Sets the set_fields on any record with matching match_fields int table table_name
   *   @param   table_name   Name of the sql table
   *   @param   set_fields   The fields to set
   *   @param   match_fields An array of fields to use in a WHERE clause
   *   @throws  e            Any database exception.
   *   @todo                 Nothing
   **/

  function SetFieldsByFields( $table_name, $set_fields, $match_fields )
  {
    if( count( $set_fields ) < 2 || count( $match_fields ) < 2 )
      throw MysqlDBException( 'SetFieldsByFields: Need atleast 2 args to set fields and 2 to match fields.', OUT_OF_RANGE );

    $update_sql = ' UPDATE ' . $table_name . ' SET ';

    for( $i = 0; $i < count( $set_fields ); $i += 2 )
    {
      if( 0 != $i )
        $update_sql .= ', ';

      $update_sql .= $set_fields[$i] . ' = ';
      if( !is_numeric( $set_fields[$i+1] ) )
        $update_sql .= '\'';

      $update_sql .= $set_fields[$i+1];
      if( !is_numeric( $set_fields[$i+1] ) )
        $update_sql .= '\'';
    }

    $update_sql .= ' WHERE ';

    for( $i = 0; $i < count( $match_fields ); $i += 2 )
    {
      if( $i != 0 )
        $update_sql .= ' AND ';
      $update_sql .= $match_fields[$i] . ' = ';

      if( !is_numeric( $match_fields[$i+1] ) )
        $update_sql .= '\'';

      $update_sql .= $match_fields[$i+1];

      if( !is_numeric( $match_fields[$i+1] ) )
        $update_sql .= '\'';
    }

    if( $this->debug )
      echo $update_sql;

    $this->DoQuery( $update_sql );
  }



  /**
   *   @brief   Inserts a new record into the table given with the field values given
   *   @param   table_name  The table to insert the fields into
   *   @param   field_specs An array(not associative) of field/value pairs to create the new record from.
   *   @throws  e           Any database exception.
   *   @example usage       InsertWithFields( 'users', Array( 'name', 'dave', 'email', 'nopeer@gmail.com' ) );
   **/

  function InsertWithFields( $table_name, $field_specs )
  {
    $insert_sql = ' INSERT INTO ' . $table_name . ' ( ';

    for( $i = 0; $i < count( $field_specs ); $i += 2 )
    {
      if( $i != 0 )
        $insert_sql .= ', ';

      $insert_sql .= $field_specs[$i];
    }

    $insert_sql .= ' ) VALUES ( ';

    for( $i = 1; $i < count( $field_specs ); $i += 2 )
    {
      if( $i != 1 )
        $insert_sql .= ', ';

      $insert_sql .= '\'' . $field_specs[$i] . '\'';
    }
    $insert_sql .= ' ) ';

    if( $this->debug )
      echo $insert_sql . '<br>';

    $this->DoQuery( $insert_sql );
  }



  /**
   *   @brief   Insert the given associative array as a new record into the table
   *   @param   table_name  The table to insert the new row into
   *   @param   tuple       An associative array representing the record
   *   @throws  e           Any database exception.
   *   @example usage       InsertTupleUsingAssoc( 'users', Array( 'name' => 'dave', 'email' => 'nopeer@gmail.com' ) );
   **/

  function InsertTupleUsingAssoc( $table_name, $tuple )
  {
    $insert_sql = ' INSERT INTO ' . $table_name . ' ( ';

    $num_fields = count( $tuple );
    $ind = 1;
    foreach( $tuple as $next_name => $next_value )
    {
      $insert_sql .= $next_name . ' ';
      if( $ind < $num_fields )
        $insert_sql .= ', ';
      $ind ++;
    }
    $insert_sql .= ' ) VALUES ( ';

    $ind = 1;
    foreach( $tuple as $next_name => $next_value )
    {
      if( is_string( $next_value ) || $next_value == null )
        $insert_sql .= '"' . $next_value . '" ';
      else
        $insert_sql .= $next_value;
      if( $ind < $num_fields )
        $insert_sql .= ', ';
      $ind ++;
    }
    $insert_sql .= ' ) ';

    if( $this->debug )
      echo $insert_sql . '<br>';

    $this->DoQuery( $insert_sql );
  }

  //! turn on debugging

  function Debug()
  {
    $this->debug = 1;
  }

  //! turn off debugging

  function EndDebug()
  {
    $this->debug = 0;
  }

};








?>